/****************************************************************************
*                             SENDRCV.H
*              Header filer to SENDRCV.C, that demonstrates serial
*	       communication on a MODULAR-4/486 board using CQ8;
*	       operating system MS-DOS
*
*
*              version     : 1.E.
*	       last change : 05.11.1997
*
*              version     : 1.D.
*	       last change : 29.03.1996
*
*              version     : 1.C.
*	       creation    : 06.12.1995
*	       last change : 06.12.1995
*	       author	   : H. Klinkenborg
*
*    (c) by SORCUS Computer GmbH Heidelberg 1995
****************************************************************************/

/****************************************************************************
*  Description: This program sends a string of characters via the serial
*	 interface A of the MODULAR-4/486 board and receives this string
*	 via the serial interface B of the MODULAR-4/486 board. The CQ8-
*      	 programs M8P0500.LIB and M8P0520.LIB have to be in the subdirectory
*	 SORCUS\CQ8, this example has to be in the subdirectory
*	 SORCUS\CQ8\BEISP\C.
*
*  Compiler: Borland C 3.1
*
*  hardware requirements: PC with monitor, MODULAR-4/486 board
*
*  software requirements: This program requires the library ML8MAIN.LIB
*        and the header ML8BIB.H. The pathes of M8P0500.LIB and M8P0520.LIB
*	 are given by constant strings M8P0500 and M8P0520 in SENDRCV.H.
* 	 The operating system OSX on the MODULAR-4/486 board should be
*	 ML8-2F.05x or greater
****************************************************************************/

/****************************************************************************
*  history:
*        1.G. 15.10.1998
*
*        1.F. 26.06.1998
*	      -  Fehler in init_cq8_parameters beseitigt
*        1.E. 05.11.1997 10:01; HK
*	       - os_string is = "ROM" and not "ML8-3A.02R"
*        1.D. 29.03.1996 08:30; HK
*	       - parameters clock_send and clock_rcv added
*		 both have to be "1"
*		 (necessary if using M-COM-2)
*        1.C. 06.12.1995 13:46; HK
*	       - creation of the file
****************************************************************************/

  char os_string[80] = "ROM";	       /* Operating system on board
					  MODULAR-4/486 */
  ushort IMtask = 0x300;               /* No. of interrupt manager task for
					  the serial interfaces A and B of
					  the MODULAR-4/486 board */
  ushort interrupt_no = 0x90;          /* No. of interrupt (MODULAR-4/486
					  SCC = 90H, modules = 79H,7BH .. 7FH */

  /* Path and name of interrupt manager program file */

  char M8P0500[80] = "..\\..\\M8P0500.LIB";

  /* Path and name of basic communication program */

  char M8P0520[80] = "..\\..\\M8P0520.LIB";
  ushort transmit_ctrl_function = 5;   /* Function for transmit control */
  ushort send_string_function = 7;     /* Function for transmit strings */
  ushort receive_ctrl_function = 15;   /* Function for receive control */
  ushort receive_string_function = 17; /* Function for receive strings */
  byte error = 0;	       	       /* Error byte */
  ushort ml8_adr = 0x380;    	       /* Base address of MODULAR-4/486 */
  ushort ml8_err = 0;                  /* Error byte */

  struct {                             /* Parameter record of program
					  M8P0520 */
	  ushort IMpgm_no;             /* Pgm-no. of interrupt manager */
	  ushort IMtask_no;            /* Task-no. of interrupt manager */
	  byte slot_no;                /* Slot-no. */
	  byte channel;                /* Channel-no. */
	  byte d1[8];	   	       /* Special Parameters */
	  byte clock_send;	       /* Clock source for transmit */
	  byte clock_rcv;	       /* Clock source for receive */
	  ulong quartz;		       /* Quartz frequency */
	  ulong baudrate;	       /* Baud rate */
	  byte len;		       /* Bits per character */
	  byte start_stop;	       /* Start-/Stop-Bits */
	  byte parity;		       /* Parity */
	  } M8P0520_PAR = {0x500,0x300,0,0,0,0,0,0,0,0,0,0,1,1,0x737,9600,8,1,0};


  char out_data_str[255];              /* Parameter of function call */
  char in_data_str1[255];	       /* Parameter of function call */
  char in_data_str2[255];	       /* Parameter of function call */
  short index= 0;                      /* Index of in_data_str */
  byte first = 0;		       /* Flag: = 0: Reset ML8 */
				       /*       = 1: No Reset ML8 */

void far pascal error_handler(void);   /* Default error handler */

/****************************************************************************
*  init_cq8
*     This procedure resets the MODULAR-4/486 board and transfers and in-
*     stalls the communication programs M8P0500.LIB and M8P0520.LIB
*
*     slot_no:       slot number of communication module, (= 0: base board )
*
*     channel:       channel number for installation of communication
*		     program M8P0520.LIB (0 = interface A, 1 = interface B)
*
*     first:	     flag used for reset of MODULAR-4
*		     = 0: ML8BIB is initalized, MODULAR-4 board is reset
*			  and macros are interruptible, interrupt manager
*			  program for serial interfaces on MODULAR-4 board
*			  is transferred and installed
*		     = 1: communication program for channel is transferred
*                         and installed
*
****************************************************************************/

void init_cq8(byte slot_no, byte channel);

/****************************************************************************
*  init_cq8_parameters
*     This procedure sets some parameters of the task M8P0520.LIB
*
*     slot_no:       slot number of communication module, (= 0: base board )
*
*     channel:       channel number for communication program M8P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     quartz:        frequency of quartz on MODULAR-4 board, default
*		     7.3728 MHz
*
*     baudrate:	     baud rate, e.g. 9600 baud
*
*     len:	     bits per character, e.g. 8 bits
*
*     start_stop:    start and stop bits, e.g. 1 stop bit
*
*     parity:	     parity, e.g. no parity = 0
*
****************************************************************************/

void init_cq8_parameters(byte slot_no, byte channel, ulong quartz,
				     ulong baudrate, byte len,
				     byte start_stop, byte parity);

/****************************************************************************
*  send_string
*     This function sends a string via the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board )
*
*     channel:       channel number for communication program M8P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     s:             string to be send
*
*     return value:  error message from function call
*
****************************************************************************/

byte send_string (byte slot_no, byte channel, char *s);

/****************************************************************************
*  rcv_string
*     This function receives a string via the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board )
*
*     channel:       channel number for communication program M8P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     s:             string to be received
*
*     return value:  error message from function call
*
****************************************************************************/

byte rcv_string(byte slot_no, byte channel, char *s);

void sendrcv(void);		       /* Transmit and receive data */

/****************************************************************************
*  check_func_error
*     This procedure checks, wether an error occurs during a function call
*
*     slot_no:       slot number of communication module, (= 0: base board )
*
*     channel:	     channel number
*		     (0 = interface A, 1 = interface B)
*
*     error:         error number returned by function
*
*     func:          function number of corresponding task
*
****************************************************************************/

void check_func_error(byte slot_no, byte channel, byte error, ushort func);

